# -*- coding: utf-8 -*-

import logging
import sys
from termcolor import colored

logger = None

class ColoredStreamHandler(logging.StreamHandler):

    def format(self, record):
        message = super(ColoredStreamHandler, self).format(record)
        if record.levelno == logging.INFO:
            message = colored(message, 'green')
        elif record.levelno == logging.WARN:
            message = colored(message, 'yellow')
        return message

def get_logger(name):
    global logger
    if logger is None:
        logger = logging.getLogger(name)
        logger.setLevel(logging.INFO)
        handler = ColoredStreamHandler(stream=sys.stderr)
        handler.setLevel(logging.INFO)
        handler.setFormatter(
            logging.Formatter('%(name)s: %(message)s'))
        logger.removeHandler(handler)
        logger.addHandler(handler)
    return logger



# -*- coding: utf-8 -*-

import yaml
import os
import logging
from abc import ABCMeta, abstractmethod
from collections import OrderedDict
from termcolor import colored
from lamvery.archive import Archive
from lamvery.client import Client
from lamvery.config import Config
from lamvery.log import get_logger

CONF_DIFF_KEYS = [
    ('Runtime', 'runtime',),
    ('Role', 'role',),
    ('Handler', 'handler',),
    ('Description', 'description',),
    ('Timeout', 'timeout',),
    ('MemorySize', 'memory_size',),
]

class BaseAction:

    __metaclass__ = ABCMeta

    _logger = None

    def __init__(self, args):
        self._config = Config(args.conf_file)

        if hasattr(args, 'dry_run') and args.dry_run:
            self._logger = get_logger('(Dry run) lamvery')
        else:
            self._logger = get_logger('lamvery')

    @abstractmethod
    def action(self):
        raise NotImplementedError

    def get_client(self):
        return Client(
            region=self._config.get_region(),
            profile=self._config.get_profile())

class InitAction(BaseAction):

    def __init__(self, args):
        super(InitAction, self).__init__(args)
        self._conf_file = args.conf_file

    def action(self):
        self._logger.info('Start initialization...')
        if self._needs_write_conf():
            self._config.write_default()
            self._logger.info(
                'Output initial configuration file to {}'.format(self._conf_file))

    def _needs_write_conf(self):
        ret = True
        if self._config.file_exists():
            y_n = raw_input(
                colored('Overwrite {}? [y/n]: '.format(self._conf_file), 'yellow'))
            if y_n != 'y':
                ret = False
        return ret

class ArchiveAction(BaseAction):

    def action(self):
        self._logger.info('Start archiving...')
        archive_name = self._config.get_archive_name()
        secret = self._config.generate_lambda_secret()
        archive = Archive(archive_name, secret)
        zipfile = archive.create_zipfile()
        with open(archive_name, 'w') as f:
            f.write(zipfile.read())
        zipfile.close()
        self._logger.info('Output package zip file to {}'.format(archive_name))

class DeployAction(BaseAction):

    def __init__(self, args):
        super(DeployAction, self).__init__(args)
        self._dry_run = args.dry_run
        self._publish = args.publish
        self._set_alias = SetAliasAction(args)

    def action(self):
        self._logger.info('Start deployment...')
        archive_name = self._config.get_archive_name()
        secret = self._config.generate_lambda_secret()
        archive = Archive(archive_name, secret)
        func_name   = self._config.get_function_name()
        local_conf  = self._config.get_configuration()
        zipfile     = archive.create_zipfile()
        client      = self.get_client()
        remote_conf = client.get_function_conf(func_name)

        self._print_conf_diff(remote=remote_conf, local=local_conf)

        if not self._dry_run:
            if len(remote_conf) > 0:
                client.update_function_code(zipfile, local_conf, self._publish)
                client.update_function_conf(local_conf)
            else:
                client.create_function(zipfile, local_conf, self._publish)
        zipfile.close()
        self._set_alias.action()
        self._logger.info('Deploy finished.')

    def _get_conf_diff(self, remote, local):
        diff = {}
        for k in CONF_DIFF_KEYS:
            r = remote.get(k[0])
            l = local.get(k[1])
            if r == l:
                diff[k[1]] = None
            else:
                diff[k[1]] = (r, l,)
        return diff

    def _print_conf_diff(self, remote, local):
        diff = self._get_conf_diff(remote, local)
        for k,v in diff.items():
            if v is None:
                self._logger.warn(
                    '[Configuration] {k}: No change'.format(k=k))
            else:
                self._logger.warn(
                    '[Configuration] {k}: {r} > {l}'.format(k=k, r=v[0], l=v[1]))

class SetAliasAction(BaseAction):

    def __init__(self, args):
        super(SetAliasAction, self).__init__(args)
        self._dry_run = args.dry_run
        self._alias = args.alias
        if hasattr(args, 'alias_version'):
            self._alias_version = args.alias_version
        else:
            self._alias_version = None

    def action(self):
        self._logger.info('Start alias setting...')
        alias_name = self._get_alias_name()
        version    = self._get_alias_version()
        func_name  = self._config.get_function_name()
        client     = self.get_client()

        if alias_name is not None:
            current_alias = client.get_alias(func_name, alias_name)
            self._print_alias_diff(alias_name, current_alias, version)

            if not self._dry_run:
                if len(current_alias) > 0:
                    client.update_alias(func_name, alias_name, version)
                else:
                    client.create_alias(func_name, alias_name, version)
        self._logger.info('Finish alias setting.')

    def _print_alias_diff(self, name, current, version):
        self._logger.warn(
            '[Alias] {name}: {cur} > {new}'.format(
                name=name, cur=current.get('FunctionVersion'), new=version))

    def _get_alias_name(self):
        if self._alias is not None:
            return self._alias
        return self._config.get_configuration().get('alias')

    def _get_alias_version(self):
        if self._alias_version is None:
            return '$LATEST'
        return self._alias_version

class EncryptAction(BaseAction):

    def __init__(self, args):
        super(EncryptAction, self).__init__(args)
        self._text = args.text
        self._name = args.secret_name
        self._store = args.store

    def action(self):
        client = self.get_client()
        cipher_text = self.get_client().encrypt(
            self._config.get_secret().get('key'), self._text)

        if self._store:
            self._config.store_secret(self._name, cipher_text)
        else:
            print(cipher_text)

class DecryptAction(BaseAction):

    def __init__(self, args):
        super(DecryptAction, self).__init__(args)
        self._name = args.secret_name

    def action(self):
        client = self.get_client()
        text = self.get_client().decrypt(
            self._config.get_secret().get('cipher_texts').get(self._name))
        print(text)



# -*- coding: utf-8 -*-

import boto3
import botocore

class Client:

    def __init__(self, region=None, profile=None):
        session = boto3.session.Session(
            profile_name=profile, region_name=region)
        self._lambda = session.client('lambda')
        self._kms = session.client('kms')

    def get_function_conf(self, name):
        try:
            res = self._lambda.get_function(
                FunctionName=name)
            return res['Configuration']
        except botocore.exceptions.ClientError:
            return {}

    def create_function(self, zipfile, conf, publish):
        self._lambda.create_function(
            FunctionName=conf['name'],
            Runtime=conf['runtime'],
            Role=conf['role'],
            Handler=conf['handler'],
            Code={
                'ZipFile': zipfile.read(),
            },
            Description=conf['description'],
            Timeout=conf['timeout'],
            MemorySize=conf['memory_size'],
            Publish=publish,
        )

    def update_function_code(self, zipfile, conf, publish):
        self._lambda.update_function_code(
            FunctionName=conf['name'],
            ZipFile=zipfile.read(),
            Publish=publish,
        )

    def update_function_conf(self, conf):
        self._lambda.update_function_configuration(
            FunctionName=conf['name'],
            Role=conf['role'],
            Handler=conf['handler'],
            Description=conf['description'],
            Timeout=conf['timeout'],
            MemorySize=conf['memory_size'],
        )

    def get_alias(self, function, alias):
        try:
            return self._lambda.get_alias(
                FunctionName=function,
                Name=alias)
        except botocore.exceptions.ClientError:
            return {}

    def create_alias(self, function, alias, version):
        self._lambda.create_alias(
                FunctionName=function,
                Name=alias,
                FunctionVersion=version)

    def update_alias(self, function, alias, version):
        self._lambda.update_alias(
                FunctionName=function,
                Name=alias,
                FunctionVersion=version)

    def encrypt(self, key, text):
        res = self._kms.encrypt(KeyId=key, Plaintext=text)
        return res.get('CiphertextBlob')

    def decrypt(self, cipher_text):
        res = self._kms.decrypt(CiphertextBlob=cipher_text)
        return res.get('Plaintext')



# -*- coding: utf-8 -*-

import yaml
import os
from termcolor import cprint, colored
from collections import OrderedDict
from lamvery.archive import Archive
from lamvery.client import Client
from jinja2 import Environment, FileSystemLoader

def represent_odict(dumper, instance):
     return dumper.represent_mapping(u'tag:yaml.org,2002:map', instance.items())

yaml.add_representer(OrderedDict, represent_odict)
yaml.add_constructor(yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
    lambda loader, node: OrderedDict(loader.construct_pairs(node)))

class Config:

    def __init__(self, conf_file):
        self._file = conf_file

    def load_conf(self):
        env = Environment(loader=FileSystemLoader('./', encoding='utf8'))
        tmpl = env.get_template(self._file)
        return yaml.load(tmpl.render({'env': os.environ}))

    def get_configuration(self):
        return self.load_conf().get('configuration')

    def get_secret(self):
        return self.load_conf().get('secret')

    def generate_lambda_secret(self):
        return {
            'region': self.get_region(),
            'cipher_texts': self.get_secret().get('cipher_texts')
        }

    def get_function_name(self):
        if os.path.exists(self._file):
            return self.get_configuration().get('name')
        else:
            return os.path.basename(os.getcwd())

    def get_archive_name(self):
        return '{}.zip'.format(self.get_function_name())

    def get_region(self):
        if os.path.exists(self._file):
            return self.load_conf().get('region')
        else:
            return None

    def get_profile(self):
        return self.load_conf().get('profile')

    def get_default(self):
        init_config = OrderedDict()
        init_config['name']        = self.get_function_name()
        init_config['runtime']     = 'python2.7'
        init_config['role']        = 'arn:aws:iam::<account-number>:role/<role>'
        init_config['handler']     = 'lambda_function.lambda_handler'
        init_config['description'] = 'This is sample lambda function.'
        init_config['timeout']     = 10
        init_config['memory_size'] = 128

        init_secret = OrderedDict()
        init_secret['key'] = '<key-id>'
        init_secret['cipher_texts'] = OrderedDict()

        init_yaml = OrderedDict()
        init_yaml['profile'] = None
        init_yaml['region']  = 'us-east-1'
        init_yaml['configuration'] = init_config
        init_yaml['secret'] = init_secret

        return init_yaml

    def write_default(self):
        self.write(self.get_default())

    def write(self, conf):
        yaml.dump(
            conf,
            open(self._file, 'w'),
            default_flow_style=False,
            allow_unicode=True)

    def file_exists(self):
        return os.path.exists(self._file)

    def store_secret(self, key, value):
        conf = self.load_conf()
        default = self.get_default()

        if 'secret' not in conf:
            conf['secret'] = default['secret']
        elif 'cipher_texts' not in conf['secret']:
            conf['secret']['cipher_texts'] = {}

        conf['secret']['cipher_texts'][key] = value
        self.write(conf)



#!/usr/bin/env python
import sys

from setuptools import setup, find_packages

import lamvery

setup_options = dict(
    name='lamvery',
    version=lamvery.__version__,
    description='Yet another deploy tool for AWS Lambda in the virtualenv environment.',
    long_description='''
See: `GitHub <https://github.com/marcy-terui/lamvery/blob/master/README.md>`__
    ''',
    author='Masashi Terui',
    author_email='marcy9114+pypi@gmail.com',
    url='https://github.com/marcy-terui/lamvery',
    packages=find_packages(exclude=['tests*']),
    install_requires=open('requirements.txt').read().splitlines(),
    entry_points={
    'console_scripts':
        'lamvery = lamvery.cli:main'
    },
    license="MIT License",
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
    ),
    keywords='aws lambda',
)

setup(**setup_options)
